@@ -19,6 +19,8 @@
     ---------------------------------------------------------------------- */
 
 #include <config.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/gnome-ui-init.h>
@@ -69,6 +71,7 @@ static void logview_set_fontsize (LogviewWindow *logview);
 static void logview_bigger_text (GtkAction *action, GtkWidget *callback_data);
 static void logview_smaller_text (GtkAction *action, GtkWidget *callback_data);
 static void logview_normal_text (GtkAction *action, GtkWidget *callback_data);
+static void logview_calendar_set_state (LogviewWindow *LogviewWindow);
 static void logview_menus_set_state (LogviewWindow *logviewwindow);
 static void logview_search (GtkAction *action, GtkWidget *callback_data);
 static void logview_help (GtkAction *action, GtkWidget *parent_window);
@@ -262,14 +265,19 @@ static gint
 die (GnomeClient *gnome_client, gpointer client_data)
 {
     gtk_main_quit ();
+    return 0;
 }
 
 void
 logview_select_log (LogviewWindow *logview, Log *log)
 {
+	g_return_if_fail (LOGVIEW_IS_WINDOW (logview));
+	g_return_if_fail (log);
+	
 	logview->curlog = log;
 	logview_menus_set_state (logview);
-	init_calendar_data (logview);
+	logview_calendar_set_state (logview);
+	logview_update_version_bar (logview);
 	log_repaint (logview);
 	logview_save_prefs (logview);
 } 
@@ -287,7 +295,7 @@ void
 logview_add_log_from_name (LogviewWindow *logview, gchar *file)
 {
 	Log *log;
-	log = OpenLogFile (file);
+	log = OpenLogFile (file, TRUE);
 	if (log != NULL) {
 		  logview_add_log (logview, log);
 	}
@@ -301,7 +309,7 @@ logview_add_logs_from_names (LogviewWindow *logview, GSList *lognames)
 	
 	for (list = lognames; list != NULL; list = g_slist_next (list)) {
 		if (logview_find_log_from_name (logview, list->data) == NULL) {
-			log = OpenLogFile (list->data);
+			log = OpenLogFile (list->data, FALSE);
 			if (log != NULL) {
 				logview_add_log (logview, log);
 			}
@@ -309,6 +317,7 @@ logview_add_logs_from_names (LogviewWindow *logview, GSList *lognames)
 	}
 }
 
+void
 logview_remove_log (LogviewWindow *logview, Log *log)
 {
 	CloseLog (log);
@@ -453,27 +462,32 @@ loglist_selection_changed (GtkTreeSelection *selection, LogviewWindow *logview)
   gchar *name;
   Log *log;
   
+  g_return_if_fail (LOGVIEW_IS_WINDOW (logview));
+  g_return_if_fail (selection);
+
   if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		/* there is no selected log right now */
-		logview->curlog = NULL;
-		logview_set_window_title (logview);
-		logview_menus_set_state (logview);
-		gtk_calendar_clear_marks (GTK_CALENDAR(logview->calendar));
-		log_repaint (logview);
-    return;
-	}
+      /* there is no selected log right now */
+      logview->curlog = NULL;
+      logview_set_window_title (logview);
+      logview_menus_set_state (logview);
+      gtk_calendar_clear_marks (GTK_CALENDAR(logview->calendar));
+      log_repaint (logview);
+      return;
+  }
   
   gtk_tree_model_get (model, &iter, 0, &name, -1);
-	if (g_str_has_prefix (name, "<b>")) {
-		gchar *stripped;
-		stripped = g_strndup (name + 3, strlen(name)-7);
-		g_free (name);
-		name = stripped;
-	}
+  g_return_if_fail (name);
+
+  if ((g_str_has_prefix (name, "<b>")) && (strlen (name) > 7)) {
+      gchar *stripped;
+      stripped = g_strndup (name + 3, strlen(name)-7);
+      g_free (name);
+      name = stripped;
+  }
+
   log = logview_find_log_from_name (logview, name);
-	g_free (name);
-  if (log != logview->curlog)
-      logview_select_log (logview, log);
+  g_free (name);
+  logview_select_log (logview, log);
 }
 
 GtkTreePath *
@@ -599,6 +613,34 @@ loglist_create (LogviewWindow *window)
   return vbox;
 }
 
+void
+logview_version_selector_changed (GtkComboBox *version_selector, gpointer user_data)
+{
+	LogviewWindow *logview = user_data;
+	Log *log = logview->curlog;
+	int selected;
+
+	selected = gtk_combo_box_get_active (version_selector);
+
+	if (selected == log->current_version)
+		return;
+
+	/* select a new version */
+	if (selected == 0) {
+		logview_select_log (logview, log->parent_log);
+	} else {
+		Log *new;
+		if (log->parent_log) {
+			new = log->parent_log->older_logs[selected];
+		} else {
+			new = log->older_logs[selected];
+		}
+
+		logview_select_log (logview, new);
+	}
+}
+	
+
 
 /* ----------------------------------------------------------------------
    NAME:        CreateMainWin
@@ -620,6 +662,7 @@ CreateMainWin (LogviewWindow *window)
    GtkWidget *menubar;
    GtkWidget *loglist;
    GtkWidget *hpaned;
+	 GtkWidget *label;
 	 PangoFontDescription *fontdesc;
 	 PangoContext *context;
    const gchar *column_titles[] = { N_("Date"), N_("Host Name"),
@@ -681,21 +724,30 @@ CreateMainWin (LogviewWindow *window)
 	 gtk_box_pack_start (GTK_BOX(window->main_view), window->mon_scrolled_window, TRUE, TRUE, 0);
 
    /* Main Tree View */
-   tree_store = gtk_tree_store_new (4, G_TYPE_STRING, G_TYPE_STRING,
-				    G_TYPE_STRING, G_TYPE_STRING);
-
-   window->view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
+   window->view = gtk_tree_view_new ();
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->view), TRUE);
-   g_object_unref (G_OBJECT (tree_store)); 
    
    for (i = 0; column_titles[i]; i++) {
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_(column_titles[i]),
                     renderer, "text", i, NULL);
-        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); 
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_append_column (GTK_TREE_VIEW (window->view), column);
    }
+
+	 /* Version selector */
+	 window->version_bar = gtk_hbox_new (FALSE, 0);
+	 gtk_container_set_border_width (GTK_CONTAINER (window->version_bar), 3);
+	 window->version_selector = gtk_combo_box_new_text ();
+	 g_signal_connect (G_OBJECT (window->version_selector), "changed",
+										 G_CALLBACK (logview_version_selector_changed), window);
+	 label = gtk_label_new (_("Version : "));
+
+	 gtk_box_pack_end (GTK_BOX(window->version_bar), window->version_selector, FALSE, FALSE, 0);
+	 gtk_box_pack_end (GTK_BOX(window->version_bar), label, FALSE, FALSE, 0);
+	 gtk_box_pack_end (GTK_BOX(window->main_view), window->version_bar, FALSE, FALSE, 0);
+
 	 /* Remember the original font size */
 	 context = gtk_widget_get_pango_context (window->view);
 	 fontdesc = pango_context_get_font_description (context);
@@ -711,13 +763,14 @@ CreateMainWin (LogviewWindow *window)
    /* Add signal handlers */
    g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (handle_selection_changed_cb), window);
-   g_signal_connect (G_OBJECT (window->view), "row_activated",
-                     G_CALLBACK (handle_row_activation_cb), window);
+	 g_signal_connect (G_OBJECT (window->view), "row-expanded",
+										 G_CALLBACK (handle_row_expansion_cb), window);
+	 g_signal_connect (G_OBJECT (window->view), "row-collapsed",
+										 G_CALLBACK (handle_row_collapse_cb), window);
    g_signal_connect (G_OBJECT (window), "configure_event",
 		     G_CALLBACK (window_size_changed_cb), window);
 
    window->find_bar = logview_findbar_new (window);
-	 //   gtk_toolbar_set_style (GTK_TOOLBAR (window->find_bar), GTK_TOOLBAR_BOTH_HORIZ);
 	 gtk_box_pack_start (GTK_BOX (vbox), window->find_bar, FALSE, FALSE, 0);
    gtk_widget_show (window->find_bar);
 
@@ -728,6 +781,7 @@ CreateMainWin (LogviewWindow *window)
 	 gtk_widget_show_all (vbox);
    gtk_widget_hide (window->find_bar);
 	 gtk_widget_hide (window->mon_scrolled_window);
+	 gtk_widget_hide (window->version_bar);
 }
 
 /* ----------------------------------------------------------------------
@@ -781,7 +835,7 @@ FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 	   }
 
 	   Log *tl;
-	   if ((tl = OpenLogFile (f)) != NULL)
+	   if ((tl = OpenLogFile (f, TRUE)) != NULL)
 	     logview_add_log (logview, tl);
    }
    
@@ -1003,6 +1057,7 @@ logview_set_fontsize (LogviewWindow *logview)
 {
 	PangoFontDescription *fontdesc;
 	PangoContext *context;
+	int i;
 	
 	context = gtk_widget_get_pango_context (logview->view);
 	fontdesc = pango_context_get_font_description (context);
@@ -1088,31 +1143,49 @@ logview_menu_item_set_state (LogviewWindow *logviewwindow, char *path, gboolean
 	gtk_action_set_sensitive (action, state);
 }
 
+static void
+logview_calendar_set_state (LogviewWindow *logview)
+{
+	if (logview->curlog->has_date) {
+		init_calendar_data (logview);
+		if (logview->calendar_visible)
+			gtk_widget_show (logview->calendar);
+	} else {
+		gtk_widget_hide (logview->calendar);
+	}
+}
+
 static void
 logview_menus_set_state (LogviewWindow *window)
 {
 	if (window->curlog && window->curlog->monitored) {
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", FALSE);
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", FALSE);
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
 		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", FALSE);
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", FALSE);
 	} else {
 		if (window->curlog) {
+
 			if (window->curlog->display_name)
 				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
 			else
 				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", TRUE);
+
+            if (window->curlog->has_date)
+                logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
+            else
+                logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
+
 		} else
 			logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
 		
 		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/Properties", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/CloseLog", (window->curlog != NULL));
-		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", (window->curlog != NULL));
 		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", (window->curlog != NULL));
+        logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
 	}
 }
 
